
#################################################################################################
#														
# TITLE:			    Meta-analysis of anti-CCP in R using glmer		
# AUTHOR:			    Christopher Partlett 										
# DATE CREATED:		25/08/2020								   				
# DATE MODIFIED:	26/09/2021								   						
# PURPOSE: 			  Bivariate model for meta-analysis of anti-CCP	for RA
#									 							
# ACKNOWLEDGEMENT:	Most of the code here has been modelled on previous work by  
# 				Yemisi Takwoingi for the equivalent Stata programs  								
#																
#################################################################################################


## Set your working directory to the appropriate drive where you saved the file anti-ccp.csv. Replace "U:/Handbook 2020" with your path. 
setwd("U:/Handbook 2020")

############################ 1. DATA IMPORT #####################################################
### Read in the data from the .csv file. ##
(X=read.csv("anti-ccp.csv"))


############################ 2. META-ANALYSIS OF DTA - USING GLMER ##############################

### Install lme4 package if required (to run remove the # and select an appropriate CRAN mirror).
# install.packages("lme4")

### Load the package lme4. 
library(lme4)

### In order to specify the generalized linear model, first, we need to set up the data. 
### Generate 5 new variables of type long. We need these before we can reshape the data.
# n1 is number diseased
# n0 is number without disease
# true1 is number of true positives
# true0 is the number of true negatives
# recordid is the unique identifier for each observation in the dataset 

X$n1 <- X$TP+X$FN
X$n0 <- X$FP+X$TN
X$true1 <- X$TP
X$true0 <- X$TN 

X$recordid <- 1:37

### Reshape the data from wide to long format. ###
Y = reshape(X, direction="long", varying=list(c("n1", "n0"), c("true1", "true0")), 
            timevar="sens", times=c(1,0), v.names=c("n","true")) 

### Sort data by study to cluster the 2 records per study together. ###
Y = Y[order(Y$id),]  
Y$spec<- 1-Y$sens

#############################
### Perform meta-analysis ### 
#############################
## Now, let's examine the model specification and output in more detail. 
# The variable true specifies the response while sens and spec are dummy variables. 
# The fixed effect for logit sensitivity and logit specificity are the coefficients of sens and spec. 
# The constant term is supressed by adding (0 + ...) to the model formula.  
# Adding (0 + sens + spec | study) to the model includes study-level random effects. 
# family = binomial specifies the data are in binomial form. 
# Specified in the form above, the between study covariance matrix is unstructured. 
# nAGQ controls number of points per axis for evaluating the adaptive Gauss-Hermite approximation to the log-likelihood. Defaults to 1, corresponding to the Laplace approximation. 

(MA_Y = glmer(formula=cbind(true, n - true) ~ 0 + sens + spec + (0+sens + spec|Study.ID), 
               data=Y, family=binomial, nAGQ=1, verbose=2)) 

### More detail can be obtained by using the summary command. 
(ma_Y = summary(MA_Y)) 

### To obtain the between study covariance between logit sensitivity and specificity for each test use 
(summary(MA_Y))$vcov

### For the full list of outputs 
labels( ma_Y) 

### Therefore, to extract the coefficients 
ma_Y$coeff 

(lsens = ma_Y$coeff[1,1])
(lspec = ma_Y$coeff[2,1]) 

se.lsens = ma_Y$coeff[1,2]
se.lspec = ma_Y$coeff[2,2] 

### Then we can manually create 95% confidence intervals for logit sens and spec
Sens = c(lsens, lsens-qnorm(0.975)*se.lsens, lsens+qnorm(0.975)*se.lsens) 
Spec = c(lspec, lspec-qnorm(0.975)*se.lspec, lspec+qnorm(0.975)*se.lspec) 

### Or as a dataframe 
logit_sesp = data.frame(estimate = c(lsens, lspec), 
                      lci = c(lsens-qnorm(0.975)*se.lsens, lspec-qnorm(0.975)*se.lspec), 
                      uci = c(lsens+qnorm(0.975)*se.lsens, lspec+qnorm(0.975)*se.lspec),
                      row.names = c("lSens", "lSpec")) 

### R has a built in logit and inv.logit function (use qlogis and plogis). 
plogis(Sens) 
plogis(Spec) 

### Obtaining diagnostic odds rato (DOR), positive likelihood ratio (LRp) and negative likelihood ratio (LRn) 
(DOR = exp(lsens+lspec)) 
(LRp = plogis(lsens)/(1-plogis(lspec))) 
(LRn = ((1-plogis(lsens))/plogis(lspec))) 

### Standard errors and confidence intervals of DOR and LRs can be calculated using delta method. This requires the package msm. 
# install.packages("msm")

library(msm)

se.logDOR = deltamethod (~ (x1+x2), mean=c(lsens,lspec), cov=ma_Y$vcov)

se.logLRp = deltamethod (~ log((exp(x1)/(1+exp(x1)))/(1-(exp(x2)/(1+exp(x2))))), 
                         mean=c(lsens,lspec), cov=ma_Y$vcov)

se.logLRn = deltamethod (~ log((1-(exp(x1)/(1+exp(x1))))/(exp(x2)/(1+exp(x2)))), 
                         mean=c(lsens,lspec), cov=ma_Y$vcov) 

data.frame(estimate = c(DOR, LRp, LRn), 
            lci = c(exp(log(DOR)-qnorm(0.975)*se.logDOR), exp(log(LRp)-qnorm(0.975)*se.logLRp), exp(log(LRn)-qnorm(0.975)*se.logLRn)), 
            uci = c(exp(log(DOR)+qnorm(0.975)*se.logDOR), exp(log(LRp)+qnorm(0.975)*se.logLRp), exp(log(LRn)+qnorm(0.975)*se.logLRn)),
            row.names = c("DOR", "LR+", "LR-")) 






