library(rjags)
library(DTAplots)
library(mcmcplots)



# Bayesiam HSROC model (BOX 11.12)
modelString =
"model {

# === LIKELIHOOD === #

for(i in 1:n) {		

TP[i] ~ dbin(TPR[i],pos[i])
FP[i] ~ dbin(FPR[i],neg[i])

se[i] <- TPR[i]
sp[i] <- 1-FPR[i]

# === HIERARCHICAL PRIOR FOR TPR AND FPR === #

logit(TPR[i]) <- (theta[i] + 0.5*alpha[i])/exp(beta/2)
logit(FPR[i]) <- (theta[i] - 0.5*alpha[i])*exp(beta/2)

theta[i] ~ dnorm(THETA,prec[2])
alpha[i] ~ dnorm(LAMBDA,prec[1]) 

}

### === HYPER PRIOR DISTRIBUTIONS === ###

THETA ~ dunif(-10,10) 
LAMBDA ~ dunif(-2,20)
beta ~ dunif(-5,5) 

for(i in 1:2) {
prec[i] ~ dgamma(2.1,2)
tau.sq[i] <- 1/prec[i]
tau[i] <- pow(tau.sq[i],0.5)
}	
"

writeLines(modelString,con="model.txt")

# Data for the rheumatoid factor example

TP=c(64, 482, 36, 143, 80, 61, 27, 60, 261, 20, 42, 18, 8, 93, 143, 
     84, 30, 32, 70, 48, 75, 64, 32, 130, 50, 20, 77, 73, 36, 56, 
     115, 49, 22, 8, 35, 161, 80, 5, 57, 383, 89, 57, 196, 163, 75, 
     157, 26, 62, 113, 25)
FP=c(16, 2, 6, 43, 50, 36, 6, 8, 54, 2, 46, 3, 8, 28, 39, 41, 2, 
     29, 39, 1, 42, 18, 10, 8, 14, 32, 16, 22, 3, 11, 53, 23, 2, 8, 
     8, 89, 28, 4, 9, 38, 3, 25, 75, 10, 21, 287, 1, 11, 19, 1)
FN=c(27, 82, 41, 53, 39, 37, 3, 20, 63, 29, 14, 31, 0, 25, 63, 56, 
     23, 3, 36, 40, 12, 29, 9, 128, 20, 26, 52, 29, 5, 46, 67, 28, 
     20, 31, 51, 7, 69, 11, 33, 166, 9, 6, 99, 28, 21, 78, 32, 114, 
     29, 14)
TN=c(153, 153, 313, 196, 45, 196, 33, 119, 197, 18, 127, 25, 31, 
     118, 130, 90, 73, 13, 93, 99, 191, 73, 13, 113, 191, 25, 52, 
     90, 70, 87, 63, 359, 80, 91, 149, 360, 284, 49, 93, 170, 39, 
     111, 345, 140, 106, 1466, 29, 127, 481, 20)

pos= TP + FN
neg= TN + FP 

n <- length(TP) #  Number of studies
dataList = list(TP=TP,FP=FP,n=n,pos=pos,neg=neg)

# Compile the model 
jagsModel = jags.model("model.txt",data=dataList,n.chains=3)

# Burn-in iterations 
update(jagsModel,n.iter=25000)

# Parameters to be monitored
parameters = c( "THETA", "LAMBDA", "beta", "prec", "tau.sq", "tau") 

# Posterior samples
output = coda.samples(jagsModel,variable.names=parameters,n.iter=20000)

# Plots for evaluating convergence
tiff("Figure 11.6.tiff",width = 23, height = 23, units = "cm", res=600)
par(oma=c(0,0,3,0))
layout(matrix(c(1,2,3,3), 2, 2, byrow = TRUE))
denplot(output, parms=c("LAMBDA"), auto.layout=FALSE, main="(a)", xlab="LAMBDA", ylab="Density")
rmeanplot(output, parms=c("LAMBDA"), auto.layout=FALSE, main="(b)")
title(xlab="Iteration", ylab="Running mean")
traplot(output, parms=c("LAMBDA"), auto.layout=FALSE, main="(c)")
title(xlab="Iteration", ylab="LAMBDA")
mtext("Diagnostics for LAMBDA", side=3, line=1, outer=TRUE, cex=2)
dev.off()

gelman.diag(output)

# summary statistics
summary(output) 

# Posterior density plots
denplot(output)

# SROC plot
SROC_rjags(X=output, model="HSROC",n=n, study_col1="blue", study_col2=rgb(0, 0, 1, 0.15), dataset=cbind(TP,FP,FN,TN), ref_std=FALSE,SROC_curve = T, cex.summary.point = 0, cred_region=F, predict_region = F)
