library(rjags)
library(mcmcplots)
library(MCMCvis)


modelString =
  "model {
#=================================================================
# LIKELIHOOD	
#=================================================================
for(i in 1:N) {	
# The 4 cells in the two-by-two table of the new test vs. the imperfect reference test in each study in the meta-analysis	
cell[i,1:4] ~ dmulti(prob[i,1:4],n[i])
# Multinomial probabilities of the 4 cells of the two-by-two table expressed in terms of the disease prevalence (prev), the sensitivity (se1, se2) and specificity (sp1, sp2) of the two tests and the covariance between them among disease positive (covs12) and disease negative (covc12) groups
se[i] <- p[1,i]
sp[i] <- 1-p[2,i]		
prob[i,1] <- prev[i]*( p[1,i]     * se2[i]     + covp[i] ) + (1-prev[i])*( p[2,i]     *(1-sp2[i]) + covn[i] )
prob[i,2] <- prev[i]*( p[1,i]     * (1-se2[i]) - covp[i] ) + (1-prev[i])*( p[2,i]     *sp2[i]     - covn[i] )
prob[i,3] <- prev[i]*( (1-p[1,i]) * se2[i]     - covp[i] ) + (1-prev[i])*( (1-p[2,i]) *(1-sp2[i]) - covn[i] )
prob[i,4] <- prev[i]*( (1-p[1,i]) * (1-se2[i]) + covp[i] ) + (1-prev[i])*( (1-p[2,i]) *sp2[i]     + covn[i] )

#=================================================================
# CONDITIONAL DEPENDENCE	
#=================================================================
#=======================================
# upper limits of covariance parameters
#=======================================
us[i]<-min(se[i],se2[i])-(se[i]*se2[i]);
uc[i]<-min(sp[i],sp2[i])-(sp[i]*sp2[i]);
ls[i]<- -(1-se[i])*(1-se2[i])
lc[i]<- -(1-sp[i])*(1-sp2[i])
#==============================================================
# prior distribution of transformed covariances on (0,1) range
#==============================================================
covp[i]~dunif(ls[i],us[i]);
covn[i]~dunif(lc[i],uc[i]);
 #=================================================================
# HIERARCHICAL PRIORS	
#=================================================================
#=======================================
# Hierarchichal prior for xpert
#=======================================
logit(p[1, i]) <- l[i,1]
logit(p[2, i]) <- -l[i,2]
l[i,1:2] ~ dmnorm(mu[], T[,])
#=======================================
# Prior distribution on prevalence
#=======================================
prev[i] ~ dbeta(1,1)
}

# ==================================	
# Hierarchical prior for CULTURE
# ==================================	
for(j in 1:N) {
logit(se2[j]) <- l2[j,1]
logit(sp2[j]) <- l2[j,2]
l2[j,1:2] ~ dmnorm(mu2[1:2], T2[1:2,1:2])			
}

#=================================================================
############  HYPER PRIOR DISTRIBUTIONS    ################
#=================================================================
### 			XPERT TEST
#=================================================================

### prior for the logit transformed sensitivity (mu[1]) and specificity (mu[2])
mu[1] ~ dnorm(0,0.25)
mu[2] ~ dnorm(0,0.25) 
T[1:2,1:2]<-inverse(TAU[1:2,1:2])

#### BETWEEN-STUDY VARIANCE-COVARIANCE MATRIX
TAU[1,1] <- tau[1]*tau[1]
TAU[2,2] <- tau[2]*tau[2]
TAU[1,2] <- rho*tau[1]*tau[2]	
TAU[2,1] <- rho*tau[1]*tau[2]	

#### prec = between-study precision in the logit(sensitivity) and logit(specificity)
prec[1] ~ dgamma(2,0.5)
prec[2] ~ dgamma(2,0.5)
rho ~ dunif(-1,1)

####################### OTHER PARAMETERS OF INTEREST
#### SUMMARY SENSITIVITY AND SPECIFICITY OF XPERT
Summary_Se<-1/(1+exp(-mu[1]))
Summary_Sp<-1/(1+exp(-mu[2]))

#### BETWEEN_STUDY VARIANCE IN THE LOGIT(SENSITIVITY) AND LOGIT(SPECIFICITY)
tau.sq[1] <- pow(tau[1], 2)
tau.sq[2] <- pow(tau[2], 2)

#### BETWEEN_STUDY STANDARD DEVIATION IN THE LOGIT(SENSITIVITY) AND LOGIT(SPECIFICITY)
tau[1]<-pow(prec[1],-0.5)
tau[2]<-pow(prec[2],-0.5)

#### PREDICTED SENSITIVITY AND SPECIFICITY OF XPERT IN A FUTURE STUDY
Predicted.l[1:2] ~ dmnorm(mu[],T[,])
Predicted_Se <- 1/(1+exp(-Predicted.l[1]))
Predicted_Sp <- 1/(1+exp(-Predicted.l[2]))

#=================================================================
### 			CULTURE TEST
#=================================================================

### prior for the logit transformed sensitivity (mu2[1]) and specificity (mu2[2])
mu2[1] ~ dnorm(0,0.25)
mu2[2] ~ dnorm(0,0.25)
T2[1:2,1:2]<-inverse(TAU2[1:2,1:2])

#### BETWEEN-STUDY VARIANCE-COVARIANCE MATRIX
TAU2[1,1] <- tau2[1]*tau2[1]
TAU2[2,2] <- tau2[2]*tau2[2]
TAU2[1,2] <- rho2*tau2[1]*tau2[2]	
TAU2[2,1] <- rho2*tau2[1]*tau2[2]

#### prec = between-study precision in the logit(sensitivity) and logit(specificity)
prec2[1] ~ dgamma(2,0.5)
prec2[2] ~ dgamma(2,0.5)
rho2 ~ dunif(-1,1)

#=================================================================
# OTHER PARAMETERS OF INTEREST
#=================================================================

#### SUMMARY SENSITIVITY AND SPECIFICITY OF CULTURE
Summary_Se2<-1/(1+exp(-mu2[1]))
Summary_Sp2<-1/(1+exp(-mu2[2]))

#### BETWEEN_STUDY STANDARD DEVIATION IN THE LOGIT(SENSITIVITY) AND LOGIT(SPECIFICITY)
tau2[1] <-pow(prec2[1],-0.5)
tau2[2] <-pow(prec2[2],-0.5)

#### BETWEEN_STUDY VARIANCE IN THE LOGIT(SENSITIVITY) AND LOGIT(SPECIFICITY)
tau.sq2[1] <- pow(tau2[1], 2)
tau.sq2[2] <- pow(tau2[2], 2)

#### PREDICTED SENSITIVITY AND SPECIFICITY OF CULTURE IN A FUTURE STUDY
Predicted_Se2 <- 1/(1+exp(-Predicted.ls2))
Predicted_Sp2 <- 1/(1+exp(-Predicted.lc2))
Predicted.ls2 ~ dnorm(mu2[1],prec2[1])
Predicted.lc2 ~ dnorm(mu2[2],prec2[2])		
}
"

writeLines(modelString,con="model.txt")

# Data for Tuberculosis example
n11 <- c(7, 6, 2, 5, 0, 1, 4, 3, 3, 1, 7, 35, 103, 1, 2, 22, 6, 25, 
         27, 15, 5, 2, 2, 11, 2, 0, 8, 3, 1)
n10 <- c(5, 4, 0, 0, 0, 0, 6, 0, 3, 0, 0, 5, 6, 1, 0, 1, 1, 3, 11, 3, 
         9, 3, 0, 2, 4, 0, 2, 0, 2)
n01 <- c(5, 4, 0, 1, 2, 0, 8, 1, 1, 0, 0, 1, 18, 2, 0, 5, 0, 20, 25, 
        7, 10, 1, 1, 2, 2, 3, 5, 0, 0) 
n00 <- c(63, 115, 2, 44, 132, 4, 83, 250, 67, 14, 150, 119, 252, 107, 
         8, 31, 148, 687, 204, 205, 115, 53, 4, 118, 22, 16, 186, 28, 
         43)

cell<-cbind(n11, n10, n01, n00)
N <- length(cell[,1]) #  Number of studies
n <- rowSums(cell) #  Study sample sizes
dataList = list(cell=cell,N=N, n=n)

#Initial values
initsList = list(
  list( mu = c(0,0), mu2 = c(0,0)),
  list( mu = c(rnorm(1,0,2),rnorm(1,0,2)), mu2 = c(rnorm(1,0,2),rnorm(1,0,2))),
  list( mu = c(rnorm(1,0,2),rnorm(1,0,2)), mu2 = c(rnorm(1,0,2),rnorm(1,0,2))),
  list( mu = c(rnorm(1,0,2),rnorm(1,0,2)), mu2 = c(rnorm(1,0,2),rnorm(1,0,2))),
  list( mu = c(rnorm(1,0,2),rnorm(1,0,2)), mu2 = c(rnorm(1,0,2),rnorm(1,0,2))) )

# Compile the model 
jagsModel = jags.model("model.txt",data=dataList,n.chains=5,inits=initsList, n.adapt=0)

# Burn-in iterations 
update(jagsModel,n.iter=30000)

# Parameters to be monitored
parameters = c( "prev", "se","sp", "Summary_Se", "Summary_Se2", "Summary_Sp", "Summary_Sp2") 

# Posterior samples
output = coda.samples(jagsModel,variable.names=parameters,n.iter=300000, thin=10)

# Plots to check convergence
for(i in c(1,2,10,11)) {  
  tiff(paste(parameters[i],".tiff",sep=""),width = 23, height = 23, units = "cm", res=200)  
  par(oma=c(0,0,3,0))  
  layout(matrix(c(1,2,3,3), 2, 2, byrow = TRUE))  
  denplot(output, parms=c(paste(parameters[i],sep="")), auto.layout=FALSE, main="(a)", xlab=paste(parameters[i],sep=""), ylab="Density")  
  rmeanplot(output, parms=c(paste(parameters[i],sep="")), auto.layout=FALSE, main="(b)")  
  title(xlab="Iteration", ylab="Running mean")  
  traplot(output, parms=c(paste(parameters[i],sep="")), auto.layout=FALSE, main="(c)")  
  title(xlab="Iteration", ylab=paste(parameters[i],sep=""))  
  mtext(paste("Diagnostics for ", parameters[i],"",sep=""), side=3, line=1, outer=TRUE, cex=2)  
  dev.off()  
}

gelman.diag(output)


# summary statistics
summary(output[[1]]) 

# Posterior density plots
denplot(output, parms=c("Summary_Se","Summary_Sp"))

# SROC plot
par(mfrow=c(1,2))
SROC_rjags(X=output_LC, model="Bivariate",n=N, study_col1="blue", study_col2=rgb(0, 0, 1, 0.15), dataset=cell, ref_std=FALSE,SROC_curve = F, title="Latent Class meta-analysis", Sp.range=c(0.8,1))
mtext("Latent Class meta-analysis", side=3, line=1, outer=FALSE, cex=1.25)

SROC_rjags(X=output_Bivariate, model="Bivariate",n=N, study_col1="blue", study_col2=rgb(0, 0, 1, 0.15), dataset=cbind(TP,FP,FN,TN), ref_std=TRUE,SROC_curve = F, title="Standard bivariate meta-analysis",  Sp.range=c(0.8,1))
mtext("Standard Bivariate meta-analysis", side=3, line=1, outer=FALSE, cex=1.25)

