library(rjags)
library(mcmcplots)

modelString =
  "model {
# === LIKELIHOOD === #
for(i in 1:n) {		
TP[i] ~ dbin(TPR[i],pos[i])
FP[i] ~ dbin(FPR[i],neg[i])
se[i] <- TPR[i]
sp[i] <- 1-FPR[i]
# === HIERARCHICAL PRIOR FOR TPR AND FPR === #
logit(TPR[i]) <- (theta[i] + 0.5*alpha[i])/b[i]
logit(FPR[i]) <- (theta[i] - 0.5*alpha[i])*b[i]
t[i] <- THETA + Z1[i]*gamma[1] + Z2[i]*gamma[2]
l[i] <- LAMBDA + Z1[i]*epsilon[1] + Z2[i]*epsilon[2]
b[i] <- exp((beta + Z1[i]*delta[1] + Z2[i]*delta[2])/2)
theta[i] ~ dnorm(t[i],prec[1])
alpha[i] ~ dnorm(l[i],prec[2]) 
}

### === HYPER PRIOR DISTRIBUTIONS === ###
THETA ~ dunif(-10,10) 
LAMBDA ~ dunif(-2,20)
beta ~ dunif(-5,5) 
for(j in 1:2) {
gamma[j] ~ dnorm(0, 0.01)#dunif(-10,10)
epsilon[j] ~ dnorm(0, 0.01)#dunif(-2,20)
delta[j] ~ dnorm(0, 0.01)#dunif(-5,5)
prec[j] ~ dgamma(2.1,2)
tau.sq[j] <- 1/prec[j]
tau[j] <- pow(tau.sq[j],0.5)
}	

### === PARAMETERS OF INTEREST === ###
THETA_Nephelometry <- THETA + gamma[1]
LAMBDA_Nephelometry <- LAMBDA + epsilon[1]
beta_Nephelometry <- beta + delta[1]

THETA_ELISA <- THETA + gamma[2]
LAMBDA_ELISA <- LAMBDA + epsilon[2]
beta_ELISA <- beta + delta[2]
}
"

writeLines(modelString,con="model.txt")

# Data for the rheumatoid factor example
TP=c(64, 482, 36, 143, 80, 61, 27, 60, 261, 20, 42, 18, 8, 93, 143, 
     84, 30, 32, 70, 48, 75, 64, 32, 130, 50, 20, 77, 73, 36, 49, 
     22, 8, 35, 161, 80, 5, 57, 383, 89, 57, 196, 163, 75, 157, 26, 
     62, 113)
FP=c(16, 2, 6, 43, 50, 36, 6, 8, 54, 2, 46, 3, 8, 28, 39, 41, 2, 
     29, 39, 1, 42, 18, 10, 8, 14, 32, 16, 22, 3, 23, 2, 8, 8, 89, 
     28, 4, 9, 38, 3, 25, 75, 10, 21, 287, 1, 11, 19)
FN=c(27, 82, 41, 53, 39, 37, 3, 20, 63, 29, 14, 31, 0, 25, 63, 56, 
     23, 3, 36, 40, 12, 29, 9, 128, 20, 26, 52, 29, 5, 28, 20, 31, 
     51, 7, 69, 11, 33, 166, 9, 6, 99, 28, 21, 78, 32, 114, 29)
TN=c(153, 153, 313, 196, 45, 196, 33, 119, 197, 18, 127, 25, 31, 
     118, 130, 90, 73, 13, 93, 99, 191, 73, 13, 113, 191, 25, 52, 
     90, 70, 359, 80, 91, 149, 360, 284, 49, 93, 170, 39, 111, 345, 
     140, 106, 1466, 29, 127, 481)
Z1=c(0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1, 0, 
     1, 1, 1, 1, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 0, 0, 
     0, 0, 0, 0, 0, 0)
Z2=c(0, 0, 1, 1, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 1, 
     0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1, 
     1, 1, 1, 0, 1, 0)

pos= TP + FN
neg= TN + FP 

n <- length(TP) #  Number of studies
dataList = list(TP=TP,FP=FP,n=n,pos=pos,neg=neg,Z1=Z1,Z2=Z2)

# Compile the model 
jagsModel = jags.model("model.txt",data=dataList,n.chains=3)

# Burn-in iterations 
update(jagsModel,n.iter=25000)

# Parameters to monitor
parameters = c( "THETA", "LAMBDA", "beta", "prec", "tau.sq", "gamma", "epsilon", "delta", "THETA_ELISA", "LAMBDA_ELISA", "beta_ELISA", "THETA_Nephelometry", "LAMBDA_Nephelometry", "beta_Nephelometry") 
# Posterior samples
output = coda.samples(jagsModel,variable.names=parameters,n.iter=20000)


# Plots to check convergence
for(i in c(1:3)) {  
  par(oma=c(0,0,3,0))  
  layout(matrix(c(1,2,3,3), 2, 2, byrow = TRUE))  
  denplot(output, parms=c(paste(parameters[i],sep="")),           
          auto.layout=FALSE, main="(a)", xlab=paste(parameters[i],sep=""), ylab="Density")
  rmeanplot(output, parms=c(paste(parameters[i],sep="")), auto.layout=FALSE, main="(b)")  
  title(xlab="Iteration", ylab="Running mean")  
  traplot(output, parms=c(paste(parameters[i],sep="")), auto.layout=FALSE, main="(c)")  
  title(xlab="Iteration", ylab=paste(parameters[i],sep=""))  
  mtext(paste("Diagnostics for ", parameters[i],"",sep=""), side=3, line=1, outer=TRUE, cex=2)  
  #  dev.off()
}

gelman.diag(output)
# summary statistics
summary(output) 

# Posterior density plots
denplot(output, parms=c("THETA","LAMBDA"))


