
/******************************* META-ANALYSIS OF A SINGLE INDEX TEST ****************************/

/*********** Using Proc NLMIXED for the bivariate model ******************/

/* Import data. Note you should change the file path to where you saved the anti-CCP.csv file. */
proc import out=nishimura_accp
	datafile='U:\Handbook 2020\anti-CCP.csv'
	dbms=csv replace;
	getnames=yes;
run;

proc print;
run;

/* Create two separate records for the true results in each study,the first for the diseased group, 
and the second for the non-diseased group. The variable sens is an indicator which takes the value 1 
if true=true positives and 0 otherwise, the variable spec is also an indicator that takes the value 1 
if true =true negatives and 0 otherwise. */
data nishimura_accp;
	set nishimura_accp;
	sens=1; spec=0; true=tp; n=tp+fn; output;
	sens=0; spec=1; true=tn; n=tn+fp; output;
run;

/* Ensure that both records for a study are clustered together. */
proc sort data=nishimura_accp;
	by study_id;
run;

/* Run the bivariate model with no covariates.
The "COV" option requests that a covariance matrix is printed for all model parameter estimates. 
The "ECOV" option requests a covariance matrix for all additional estimates that are computed. 
Using the PARMS statement, specify starting values for all parameters to be estimated.
Ensure that the variances of the random effects cannot be negative by using the BOUNDS statement. 
usens and uspec represent the random effects. They are both assumed to be normally distributed with mean zero. 
Their variances estimates are s2usens and s2uspec, and their covariance estimate is covesp. 
Additional estimates, e.g. the positive and negative likelihood ratios, that are functions of the model parameters 
can be estimated using the ESTIMATE statement. Take the exponent of the log LR values in the additional estimates table to get the LRs. */
proc nlmixed data=nishimura_accp cov ecov;	
	parms msens=1 to 2 by 0.5 mspec=2 to 4 by 0.5 s2usens=0.2 s2uspec=0.6 covsesp=0;
	bounds s2usens>=0;
	bounds s2uspec>=0;
	logitp = (msens + usens)*sens + (mspec + uspec)*spec;
	p = exp(logitp)/(1+exp(logitp));
	model true ~ binomial(n,p);
	random usens uspec ~ normal([0,0],[s2usens,covsesp,s2uspec])
	subject=study_id out=randeffs;
	estimate 'logLR+' log((exp(msens)/(1+exp(msens)))/(1-(exp(mspec)/(1+exp(mspec)))));
	estimate 'logLR-' log((1-(exp(msens)/(1+exp(msens))))/(exp(mspec)/(1+exp(mspec))));
run;

/* Check assumption of normality for the random effects. */
proc univariate data=randeffs plot normal;
	class effect;
	var estimate;
run;


/*********** Using MetaDAS for the bivariate model *********************/

/* The INCLUDE statement specifies the path and name of the SAS file containing the macro.*/
%include 'U:\1 Projects-Ongoing\METADAS macro\METADAS v1.3\Metadas v1.3.sas';

/* The code below uses only 4 of the input parameters available in MetaDAS to perform the meta-analysis
of anti-CCP using the bivariate model (method=b). The input parameter logfile is very useful. If the 
content of the log window is saved to a file, the tables _metadas_errors, _metadas_warnings and 
_metadas_modfail are produced and can be used for identifying problems with the model or macro instead 
of going through the entire log. */
%metadas(dtfile= 'U:\Handbook 2020\anti-CCP.csv', logfile='U:\Handbook 2020\anti-CCP log.log', method=b, 
	rfile ='U:\Handbook 2020\anti-CCP results.rtf');
run;


/**************** META-REGRESSION FOR INVESTIGATIONS OF HETEROGENEITY AND TEST COMPARISONS **************/

/* Create a dummy variable for CCP generation, coded as 0 for 'CCP1' (the referent generation)
   and coded as 1 for 'CCP2'. This new variable is added to the dataset set created above. */
data nishimura_accp; 
	set nishimura_accp;
	ccpg=0;
	if generation ="CCP2" then ccpg=1; 
run;


/***************** PART I: Model with equal variances *******************/

/* Add the covariate CCPG to the model to allow both sensitivity and specificity to be 
associated with generation of the test. Estimate logit(sensitivity) and logit(specificity) 
for CCP2 (their correlation will be output because of the "ecov" option for nlmixed), and 
also log likelihood ratios for CCP1 and CCP2. */
proc nlmixed data=nishimura_accp cov ecov;
	parms msens=1 mspec=2 s2usens=0.2 s2uspec=0.6 covsesp=0 se2=0 sp2=0;
   	bounds s2usens>=0;
   	bounds s2uspec>=0;
   	logitp=(msens+usens+se2*ccpg)*sens+(mspec+uspec+sp2*ccpg)*spec;
   	p = exp(logitp)/(1+exp(logitp)); 
   	model true ~ binomial(n,p);
   	random usens uspec ~ normal([0,0], [s2usens,covsesp,s2uspec]) 
   	subject=study_id out=randeffs;
	estimate 'logitsens CCP2'  msens + se2;
	estimate 'logitspec CCP2'  mspec + sp2;
	estimate 'logLR+ CCP1' log((exp(msens)/(1+exp(msens)))/(1-(exp(mspec)/(1+exp(mspec)))));
	estimate 'logLR- CCP1' log((1-(exp(msens)/(1+exp(msens))))/(exp(mspec)/(1+exp(mspec))));
	estimate 'logLR+ CCP2' log((exp(msens+se2)/(1+exp(msens+se2)))/(1-(exp(mspec+sp2)/(1+exp(mspec+sp2)))));
	estimate 'logLR- CCP2' log((1-(exp(msens+se2)/(1+exp(msens+se2))))/(exp(mspec+sp2)/(1+exp(mspec+sp2))));
run;

/* Check assumption of normality for the random effects. */
proc univariate data=randeffs plot normal;
  class effect;
  var estimate;
run;

/*********** Using MetaDAS for bivariate meta-regression *********************/

/* The INCLUDE statement specifies the path and name of the SAS file containing the macro. 
Do not need to run this again if included previously by running the code above. */
%include 'U:\1 Projects-Ongoing\METADAS macro\METADAS v1.3\Metadas v1.3.sas';

/* The additional options to add to the code are the ones for the covariate (covariate=generation) 
and type of effect (cveffect), i.e. covariate terms for both sensitivity and specificity (cveffect=sesp), 
for only sensitivity (cveffect=se) or for only specificity (cveffect=sp).  */
%metadas(dtfile= 'U:\Handbook 2020\anti-CCP.csv', logfile='U:\Handbook 2020\anti-CCP regression log.log', method=b, 
	cveffect=sesp, covariate = generation, rfile ='U:\Handbook 2020\anti-CCP regression results.rtf');
run;

/***************** PART II: Model with unequal variances *******************/

/*Meta-analysis of CCP1 */
proc nlmixed data=nishimura_accp cov ecov;
	parms msens=1 mspec=2 s2usens=0.5 s2uspec=0.5 covsesp=0.1;
	where ccpg=0;
   	bounds s2usens>=0;
   	bounds s2uspec>=0;
   	logitp=(msens+usens)*sens+(mspec+uspec)*spec;
   	p = exp(logitp)/(1+exp(logitp)); 
   	model true ~ binomial(n,p);
   	random usens uspec ~ normal([0,0], [s2usens,covsesp,s2uspec]) 
   	subject=study_id out=randeffs;
run;

/*Meta-analysis of CCP2 */
proc nlmixed data=nishimura_accp cov ecov;
	parms msens=1 mspec=2 s2usens=0.5 s2uspec=0.5 covsesp=0.1;
	where ccpg=1;
   	bounds s2usens>=0;
   	bounds s2uspec>=0;
   	logitp=(msens+usens)*sens+(mspec+uspec)*spec;
   	p = exp(logitp)/(1+exp(logitp)); 
   	model true ~ binomial(n,p);
   	random usens uspec ~ normal([0,0], [s2usens,covsesp,s2uspec]) 
   	subject=study_id out=randeffs;
run;

/* Easier for obtaining the required parameter estimates and variances, and also for extending 
beyond binary variables if dummy variables are created for every subgroup of the covariate. */
data nishimura_accp; 
	set nishimura_accp;
	ccpg1=0;
	ccpg2=0;
	if generation ="CCP1"  then ccpg1=1; 
	if generation ="CCP2"  then ccpg2=1; 
run;

/* Fitting model with unequal variances. 
The analysis did not converge with the default optimization technique (a quasi-Newton technique)
and so the technique was changed to the Newton-Raphson technique specified as "tech=newrap". 
Due to the change in the format of the regression equation compared to how it was specified for
model A, the ESTIMATE statements can be used to obtain the differences in the mean logits for 
sensitivity and specificity of CCP1 and CCP2 along with P values based on Wald statistics. */ 
proc nlmixed data=nishimura_accp cov ecov tech=newrap;
	parms msens1=0.1 mspec1=2 s2usens1=0.5 s2uspec1=0.5 covsesp1=0.1 
			msens2=1 mspec2=-0.2 s2usens2=0.5 s2uspec2=0.5 covsesp2=0.1;
   	bounds s2usens1>=0;
   	bounds s2uspec1>=0;
   	bounds s2usens2>=0;
   	bounds s2uspec2>=0;
   	logitp=((msens1+usens1)*ccpg1+(msens2+usens2)*ccpg2)*sens+((mspec1+uspec1)*ccpg1+(mspec2+uspec2)*ccpg2)*spec;
   	p = exp(logitp)/(1+exp(logitp)); 
   	model true ~ binomial(n,p);
    random usens1 uspec1 usens2 uspec2 ~ normal([0,0,0,0], [s2usens1,covsesp1,s2uspec1,0,0,s2usens2,0,0,covsesp2,s2uspec2]) 
   	subject=study_id out=randeffs;
	estimate 'Diff logitsens CCP2-CCP1' msens2-msens1;
	estimate 'Diff logitspec CCP2-CCP1' mspec2-mspec1;
	estimate 'logLR+ CCP1' log((exp(msens1)/(1+exp(msens1)))/(1-(exp(mspec1)/(1+exp(mspec1)))));
	estimate 'logLR- CCP1' log((1-(exp(msens1)/(1+exp(msens1))))/(exp(mspec1)/(1+exp(mspec1))));
	estimate 'logLR+ CCP2' log((exp(msens2)/(1+exp(msens2)))/(1-(exp(mspec2)/(1+exp(mspec2)))));
	estimate 'logLR- CCP2' log((1-(exp(msens2)/(1+exp(msens2))))/(exp(mspec2)/(1+exp(mspec2))));
run;

/* Check assumption of normality for the random effects. */
proc univariate data=randeffs plot normal;
  class effect;
  var estimate;
run;




