
/**************** META-REGRESSION FOR TEST COMPARISONS **************/

/* Import data. Note you should change the file path to where you saved the schuetz.csv file. */
proc import out=schuetz
	datafile='U:\Handbook 2020\schuetz.csv'
	dbms=csv replace;
	getnames=yes;
run;

/* Create a two separate records for the true results in each study, the first for the diseased group, 
and the second for the non-diseased group. The variable sens is an indicator which takes the value 1 
if true=true positives and 0 otherwise, the variable spec is also an indicator that takes the value 1 
if true =true negatives and 0 otherwise. */
data schuetz; 
	set schuetz;
	testtype=0;
	if test ="CT" then testtype=1;
	sens=1; spec=0; true=tp; n=tp+fn; output;
	sens=0; spec=1; true=tn; n=tn+fp; output;
run;

/* Ensure that both records for a study are clustered together.  */
proc sort data=schuetz;	
	by study_id test;
run;

/* Run the bivariate model with no covariates
The "cov" option requests that a covariance matrix is printed for all model parameter estimates. 
The "ecov" option requests a covariance matrix for all additional estimates that are computed. */
proc nlmixed data=schuetz cov ecov;
	parms msens=2 mspec=1 s2usens=0 s2uspec=0 covsesp=0 ;
	logitp=(msens+usens)*sens+(mspec+uspec)*spec;
	p = exp(logitp)/(1+exp(logitp)); 
	model true ~ binomial(n,p);
	random usens uspec ~ normal([0,0],[s2usens,covsesp,s2uspec]) subject=study_id out=randeffs;
run;

/* Bivariate model with test as a covariate, variances of the random effects are assumed not to vary by test type. */
proc nlmixed data=schuetz cov ecov;
	parms msens=2 mspec=1 s2usens=0 s2uspec=0 covsesp=0 se_CT=1 sp_CT=0;
	logitp=(msens+usens+se_CT*testtype)*sens+(mspec+uspec+sp_CT*testtype)*spec;
	p = exp(logitp)/(1+exp(logitp)); 
	model true ~ binomial(n,p);
	random usens uspec ~ normal([0,0],[s2usens,covsesp,s2uspec]) subject=study_id out=randeffs;
	estimate 'logitsens CT'  msens + se_CT;
	estimate 'logitspec CT'  mspec + sp_CT;
run;

/* Check assumption of normality for the random effects. */
proc univariate data=randeffs plot normal;
  class effect;
  var estimate;
run; 

/* Bivariate model with effect of test type on only sensitivity. */
proc nlmixed data=schuetz cov ecov;
	parms msens=2 mspec=1 s2usens=0 s2uspec=0 covsesp=0 se_CT=1 ;
	logitp=(msens+usens+se_CT*testtype)*sens+(mspec+uspec)*spec;
	p = exp(logitp)/(1+exp(logitp)); 
	model true ~ binomial(n,p);
	random usens uspec ~ normal([0,0],[s2usens,covsesp,s2uspec]) subject=study_id out=randeffs;
run;

/* Bivariate model with effect of test type on only specificity. */
proc nlmixed data=schuetz cov ecov;
	parms msens=2 mspec=1 s2usens=0 s2uspec=0 covsesp=0 sp_CT=0;
	logitp=(msens+usens)*sens+(mspec+uspec+sp_CT*testtype)*spec;
	p = exp(logitp)/(1+exp(logitp)); 
	model true ~ binomial(n,p);
	random usens uspec ~ normal([0,0],[s2usens,covsesp,s2uspec]) subject=study_id out=randeffs;
run;


/***************** DIRECT COMPARISONS *************************/

/* Create new dataset of studies with within-study comparison of CT and MRI. 
"indirect" is a binary variable in the dataset coded 1 if the study evaluated 
only one test (CT or MRI) and 0 if both tests were evaluated in a study. */
data schuetz_direct;
	set schuetz;
	where indirect=0;
run;

/* Fit bivariate model without covariate. */
proc nlmixed data=schuetz_direct cov ecov;
	parms msens=2 mspec=1  s2usens=0 s2uspec=0 covsesp=0;
	logitp=(msens+usens)*sens+(mspec+uspec)*spec;
	p = exp(logitp)/(1+exp(logitp)); 
	model true ~ binomial(n,p);
	random usens uspec ~ normal([0,0],[s2usens,covsesp,s2uspec]) subject=study_id out=randeffs;
run;

/* Fit bivariate model without covariate - Attempt 1: increasing the number of quadrature 
points to 10 using option qpoints. */
proc nlmixed data=schuetz_direct cov ecov qpoints=10;
	parms msens=2 mspec=1  s2usens=0 s2uspec=0 covsesp=0;
	logitp=(msens+usens)*sens+(mspec+uspec)*spec;
	p = exp(logitp)/(1+exp(logitp)); 
	model true ~ binomial(n,p);
	random usens uspec ~ normal([0,0],[s2usens,covsesp,s2uspec]) subject=study_id out=randeffs;
run;

/* Fit bivariate model without covariate - Attempt 2: changing the optimization technique from the default 
quasi-Newton technique to the Newton-Raphson technique. */
proc nlmixed data=schuetz_direct cov ecov tech=newrap;
	parms msens=2 mspec=1  s2usens=0 s2uspec=0 covsesp=0;
	logitp=(msens+usens)*sens+(mspec+uspec)*spec;
	p = exp(logitp)/(1+exp(logitp)); 
	model true ~ binomial(n,p);
	random usens uspec ~ normal([0,0],[s2usens,covsesp,s2uspec]) subject=study_id out=randeffs;
run;

/* Fit bivariate model without covariate - Attempt 3: grid search for starting values for parameters. */
proc nlmixed data=schuetz_direct cov ecov;
	parms msens=2 to 3 by 0.2 mspec=1 to 2 by 0.2  s2usens=0 to 0.2 by 0.05 s2uspec=0 to 1 by 0.1 covsesp=-0.1 to 0.1 by 0.05;
	logitp=(msens+usens)*sens+(mspec+uspec)*spec;
	p = exp(logitp)/(1+exp(logitp)); 
	model true ~ binomial(n,p);
	random usens uspec ~ normal([0,0],[s2usens,covsesp,s2uspec]) subject=study_id out=randeffs;
run;

/* Fit bivariate model without covariate - Attempt 4: set boundary constraints for the variance parameters. */
proc nlmixed data=schuetz_direct cov ecov;
	parms msens=2 mspec=1  s2usens=0 s2uspec=0 covsesp=0;
	bounds s2usens>=0;
	bounds s2uspec>=0;
	logitp=(msens+usens)*sens+(mspec+uspec)*spec;
	p = exp(logitp)/(1+exp(logitp)); 
	model true ~ binomial(n,p);
	random usens uspec ~ normal([0,0],[s2usens,covsesp,s2uspec]) subject=study_id out=randeffs;
run;

/* Fit bivariate model without covariate - Attempt 5: set boundary constraints for the variance parameters,
and change starting values for the variance parameters. */
proc nlmixed data=schuetz_direct cov ecov;
	parms msens=2 mspec=1  s2usens=0.1 s2uspec=0.1 covsesp=0;
	bounds s2usens>=0;
	bounds s2uspec>=0;
	logitp=(msens+usens)*sens+(mspec+uspec)*spec;
	p = exp(logitp)/(1+exp(logitp)); 
	model true ~ binomial(n,p);
	random usens uspec ~ normal([0,0],[s2usens,covsesp,s2uspec]) subject=study_id out=randeffs;
	estimate 'Corr(logits)' covsesp/(SQRT(s2usens)*SQRT(s2uspec)) ;
run;


/********************* SIMPLIFYING THE BIVARIATE MODEL TO UNIVARIATE MODELS **********************************/

/* Simplify the bivariate model to 2 univariate random-effects logistic regression models by  
removing the covariance parameter. */
proc nlmixed data=schuetz_direct cov ecov;
	parms msens=2 mspec=1  s2usens=0.1 s2uspec=0.1;
	bounds s2usens>=0;
	bounds s2uspec>=0;
	logitp=(msens+usens)*sens+(mspec+uspec)*spec;
	p = exp(logitp)/(1+exp(logitp)); 
	model true ~ binomial(n,p);
	random usens uspec ~ normal([0,0],[s2usens,0,s2uspec]) subject=study_id out=randeffs;
run;

/* Simplify the bivariate model further by removing the variance parameter for logit sensitivity, 
i.e. fixed effect for sensitivity. */
proc nlmixed data=schuetz_direct cov ecov;
	parms msens=2 mspec=1  s2uspec=0.1;
	bounds s2uspec>=0;
	logitp=(msens)*sens+(mspec+uspec)*spec;
	p = exp(logitp)/(1+exp(logitp)); 
	model true ~ binomial(n,p);
	random uspec ~ normal([0],[s2uspec]) subject=study_id out=randeffs;
run;

/* Fit the simplified model with covariate terms for both sensitivity and specificity.
Random effects only for specificity. */
proc nlmixed data=schuetz_direct cov ecov qpoints=10;
	parms msens=2 mspec=1  s2uspec=0.1 se_CT=0 sp_CT=0;
	bounds s2uspec>=0;
	logitp=(msens+se_CT*testtype)*sens+(mspec+uspec+sp_CT*testtype)*spec;
	p = exp(logitp)/(1+exp(logitp)); 
	model true ~ binomial(n,p);
	random uspec ~ normal([0],[s2uspec]) subject=study_id out=randeffs;	
	estimate 'logitsens CT'  msens + se_CT;
	estimate 'logitspec CT'  mspec + sp_CT;
run;

/* Fit the model with covariate terms only for specificity.
Random effects only for specificity. */
proc nlmixed data=schuetz_direct cov ecov ;
	parms msens=2 mspec=1  s2uspec=0.1 sp_CT=0;
	bounds s2uspec>=0;
	logitp=(msens)*sens+(mspec+uspec+sp_CT*testtype)*spec;
	p = exp(logitp)/(1+exp(logitp));
	model true ~ binomial(n,p);
	random uspec ~ normal([0],[s2uspec]) subject=study_id out=randeffs;	
run;

/* Fit bivariate model with covariate terms only for sensitivity.
Random effects only for specificity. */
proc nlmixed data=schuetz_direct cov ecov;
	parms msens=2 mspec=1  s2uspec=0.1 se_CT=0;
	bounds s2uspec>=0;
	logitp=(msens+se_CT*testtype)*sens+(mspec+uspec)*spec;
	p = exp(logitp)/(1+exp(logitp)); 
	model true ~ binomial(n,p);
	random uspec ~ normal([0],[s2uspec]) subject=study_id out=randeffs;
run;
