
/******************************* META-ANALYSIS OF A SINGLE INDEX TEST ****************************/

/*********** Using Proc NLMIXED for the HSROC model ******************/

/* Import data. Note you should change the file path to where you saved the RF.csv file. */
proc import out=nishimura_RF
	datafile='U:\Handbook 2020\RF.csv'
	dbms=csv
	replace;
	getnames=yes;
run;

proc print;
run;

/* Create separate records for the diseased and non-diseased groups in each study.
The variable dis is the disease indicator which takes the value 0.5 if diseased
and -0.5 if not diseased. */
data nishimura_RF;
	set nishimura_RF;
	dis=0.5; pos=tp; n=tp+fn; output;
	dis=-0.5; pos=fp; n=tn+fp; output;
run;

/* Ensure that both records for a study are clustered together. */
proc sort data=nishimura_RF;
	by study_id dis;
run;

/* Run the Rutter and Gatsonis HSROC model with no covariates. 
Request covariance matrices for model parameters ("COV"). Using the PARMS statement, 
specify starting values for all model parameters to be estimated. 
Ensure that the variances of the random effects cannot be negative by using the BOUNDS statement. 
The random effects for accuracy (ua) and threshold (ut) are assumed to be approximately normally 
distributed, both with mean zero and with variances s2ua and s2ut respectively. 
The covariance of the random effects is set to 0 in the RANDOM statement. */
proc nlmixed data=nishimura_RF ecov cov;
	/* set starting values for all model parameters to be estimated */
	parms alpha=2 theta=-1 beta=0 s2ua=0 s2ut=0;
	bounds s2ua>=0;
	bounds s2ut>=0;
	logitp = (theta + ut + (alpha + ua)*dis) * exp(-(beta)*dis);
	p = exp(logitp)/(1+exp(logitp));
	model pos ~ binomial(n,p);
	random ut ua ~ normal([0,0],[s2ut,0,s2ua]) subject=study_id out=randeffs;
run;

/* Using the ESTIMATE statement to estimate sensitivity at a fixed value of specificity. 
The ECOV option requests covariance matrices for additional estimates. 
Estimate sensitivity at fixed specificities of 80%, 87% and 93%.*/
proc nlmixed data=nishimura_RF ecov cov ;
	parms alpha=2 theta=-1 beta=0 s2ua=0 s2ut=0 ;
	bounds s2ua>=0;
	bounds s2ut>=0;
	logitp = (theta + ut + (alpha + ua)*dis) * exp(-(beta)*dis);
	p = exp(logitp)/(1+exp(logitp));
	model pos ~ binomial(n,p);
	random ut ua ~ normal([0,0],[s2ut,0,s2ua]) subject=study_id out=randeffs;
	estimate 'E(logitSe_sp80)' alpha*exp(-0.5*beta)+log(0.20/0.80)*exp(-beta);
	estimate 'E(logitSe_sp87)' alpha*exp(-0.5*beta)+log(0.13/0.87)*exp(-beta);
	estimate 'E(logitSe_sp93)' alpha*exp(-0.5*beta)+log(0.07/0.93)*exp(-beta);
run;

/*********** Using MetaDAS for the HSROC model *********************/

/* The INCLUDE statement specifies the path and name of the SAS file containing the macro.*/
%include 'U:\1 Projects-Ongoing\METADAS macro\METADAS v1.3\Metadas v1.3.sas';

/* The code below uses only 4 of the input parameters available in MetaDAS to perform the meta-analysis
of RF using the HSROC model (method=h). Note the default method in MetaDAS is the HSROC model so it is 
not necessary to state this option when fitting the HSROC model. The input parameter logfile is very useful. 
If the content of the log window is saved to a file, the tables _metadas_errors, _metadas_warnings and 
_metadas_modfail are produced and can be used for identifying problems with the model or macro instead 
of going through the entire log. */
%metadas(dtfile= 'U:\Handbook 2020\RF.csv', logfile='U:\Handbook 2020\RF log.log', method=h, 
	rfile ='U:\Handbook 2020\RF results.rtf');
run;


/**************** META-REGRESSION FOR INVESTIGATIONS OF HETEROGENEITY **************/

/* Create two dummy variables to allow for the three RF measurement methods. LA is the referent method. 
Delete the 2 studies that did not report the method, and the study that used a different method. */
data nishimura_RF; 
	set  nishimura_RF;
	if method ne "ELISA" and method ne "Nephelometry" and
	   method ne "LA" then delete;
  	rfm1=0;  rfm2=0; 	
  	if method ="ELISA" then rfm1=1;  			
  	if method ="Nephelometry" then rfm2=1;
run;

/**********PART I: variances assumed to be equal ************/

/* Include covariates to allow accuracy, threshold and shape to vary by method. */
proc nlmixed data=nishimura_RF ecov cov;
	parms alpha=2 theta=0 beta=0 s2ua=1 s2ut=1 a1=0 a2=0 t1=0 t2=0 b1=0 b2=0;
  	logitp = (theta + ut + t1*rfm1 + t2*rfm2 + (alpha + ua + a1*rfm1 + a2*rfm2)*dis)*
            exp(-(beta + b1*rfm1 + b2*rfm2)*dis);
  	p = exp(logitp)/(1+exp(logitp));
  	model pos ~ binomial(n,p);
  	random ut ua ~ normal([0,0],[s2ut,0,s2ua]) subject=study_id out=randeffs;
	/* parameter estimates for the methods of RF measurement */
  	estimate 'alpha ELISA' alpha + a1;  
	estimate 'theta ELISA' theta + t1;
  	estimate 'beta ELISA' beta + b1;
	estimate 'alpha Nephelometry' alpha + a2;
	estimate 'theta Nephelometry' theta + t2;
	estimate 'beta Nephelometry' beta + b2;
run;

/* Simplify the model to assume that all three curves have the same shape. */
proc nlmixed data=nishimura_RF ecov cov;
	parms alpha=2 theta=0 beta=0 s2ua=1 s2ut=1 a1=0 a2=0 t1=0 t2=0;
  	logitp = (theta + ut + t1*rfm1 + t2*rfm2 + (alpha + ua + a1*rfm1 + a2*rfm2)*dis)*
            exp(-(beta)*dis);
  	p = exp(logitp)/(1+exp(logitp));
  	model pos ~ binomial(n,p);
  	random ut ua ~ normal([0,0],[s2ut,0,s2ua]) subject=study_id out=randeffs;
  	estimate 'alpha ELISA'  alpha + a1;  
	estimate 'theta ELISA'  theta + t1;
	estimate 'alpha Nephelometry' alpha + a2;
	estimate 'theta Nephelometry' theta + t2;
run;

/* Check assumption of normality for random effects. */
proc univariate data=randeffs plot normal;
  class effect;
  var estimate;
run;

/* This model assumes that all three curves have the same shape and position.
The position is the same because there are no covariates included for accuracy.
Comparison with the previous model allows us to test whether accuracy varies by method. */
proc nlmixed data=nishimura_RF ecov cov;
	parms alpha=2 theta=0 beta=0 s2ua=1 s2ut=1 t1=0 t2=0;
  	logitp = (theta + ut + t1*rfm1 + t2*rfm2 + (alpha + ua)*dis)*
            exp(-(beta)*dis);
  	p = exp(logitp)/(1+exp(logitp));
  	model pos ~ binomial(n,p);
  	random ut ua ~ normal([0,0],[s2ut,0,s2ua]) subject=study_id out=randeffs;
run;

/* Check assumption of normality for random effects. */
proc univariate data=randeffs plot normal;
  class effect;
  var estimate;
run;


/***************** PART II: Model with unequal variances *******************/

/* Easier for obtaining the required parameter estimates and variances, and also for extending 
beyond binary variables if dummy variables are created for every subgroup of the covariate. */
data nishimura_RF; 
	set  nishimura_RF;
	if method ne "ELISA" and method ne "Nephelometry" and
	   method ne "LA" then delete;
  	rfm0=0;  rfm1=0; rfm2=0;	
  	if method ="LA"  then rfm0=1;
	if method ="ELISA"  then rfm1=1;  			
  	if method ="Nephelometry"  then rfm2=1;
run;

/*Meta-analysis of LA */
proc nlmixed data=nishimura_RF ecov cov;
	/* set starting values for all model parameters to be estimated */
	parms alpha=2 theta=-1 beta=0 s2ua=0 s2ut=0;
	where rfm0=1;
	bounds s2ua>=0;
	bounds s2ut>=0;
	logitp = (theta + ut + (alpha + ua)*dis) * exp(-(beta)*dis);
	p = exp(logitp)/(1+exp(logitp));
	model pos ~ binomial(n,p);
	random ut ua ~ normal([0,0],[s2ut,0,s2ua]) subject=study_id out=randeffs;
run;

/*Meta-analysis of ELISA */
proc nlmixed data=nishimura_RF ecov cov;
	/* set starting values for all model parameters to be estimated */
	parms alpha=2 theta=-1 beta=0 s2ua=0 s2ut=0;
	where rfm1=1;
	bounds s2ua>=0;
	bounds s2ut>=0;
	logitp = (theta + ut + (alpha + ua)*dis) * exp(-(beta)*dis);
	p = exp(logitp)/(1+exp(logitp));
	model pos ~ binomial(n,p);
	random ut ua ~ normal([0,0],[s2ut,0,s2ua]) subject=study_id out=randeffs;
run;

/*Meta-analysis of nephelometry */
proc nlmixed data=nishimura_RF ecov cov;
	/* set starting values for all model parameters to be estimated */
	parms alpha=2 theta=-1 beta=0 s2ua=0 s2ut=0;
	where rfm2=1;
	bounds s2ua>=0;
	bounds s2ut>=0;
	logitp = (theta + ut + (alpha + ua)*dis) * exp(-(beta)*dis);
	p = exp(logitp)/(1+exp(logitp));
	model pos ~ binomial(n,p);
	random ut ua ~ normal([0,0],[s2ut,0,s2ua]) subject=study_id out=randeffs;
run;

/* Fitting model with unequal variances. 
The analysis did not converge with the default optimisation technique (a quasi-Newton technique)
and so the technique was changed to the Newton-Raphson technique specified as "tech=newrap". 
Due to the change in the format of the regression equation compared to how it was specified for
model A, the ESTIMATE statements can be used to obtain the differences in the mean logits for 
sensitivity and specificity of CCP1 and CCP2 along with p-values based on Wald statistics. */ 

/* Include covariates to allow accuracy, threshold, shape and variance parameters to vary by method. */
proc nlmixed data=nishimura_RF ecov cov;
	parms alpha=2 theta=0 beta=0 s2ua=1 s2ut=1 a1=0 a2=0 t1=0 t2=0 b1=0 b2=0 
		s2ua1=1 s2ua2=1 s2ut1=1 s2ut2=1 covt1 = 0.1 cova1=0.1 covt2=0.1 cova2=0.1;
  	logitp = ((theta+ut)+(t1+ut1)*rfm1+(t2+ut2)*rfm2+((alpha+ua)+(a1+ua1)*rfm1+(a2+ua2)*rfm2)*dis)*
            exp(-(beta)*dis);
  	p = exp(logitp)/(1+exp(logitp));
  	model pos ~ binomial(n,p);
  	random ut ua ut1 ua1 ut2 ua2 ~ normal([0,0,0,0,0,0],[s2ut,0,s2ua,covt1,0,s2ut1,0,cova1,0,s2ua1,0,0,covt2,0,s2ut2,0,0,0,cova2,0,s2ua2]) 
		subject=study_id out=randeffs;
run;

/* Check assumption of normality for the random effects. */
proc univariate data=randeffs plot normal;
  class effect;
  var estimate;
run;

proc nlmixed data=nishimura_RF ecov cov;
	parms alpha=2 theta=0 beta=0 s2ua=1 s2ut=1 a1=0 a2=0 t1=0 t2=0 b1=0 b2=0;
  	logitp = ((theta+ut)*rfm0+(t1)*rfm1+(t2)*rfm2+((alpha+ua)*rfm0+(a1)*rfm1+(a2)*rfm2)*dis)*
            exp(-(beta*rfm0+b1*rfm1+b2*rfm2)*dis);
  	p = exp(logitp)/(1+exp(logitp));
  	model pos ~ binomial(n,p);
  	random ut ua  ~ normal([0,0],[s2ut,0,s2ua]) subject=study_id out=randeffs;
run;
