/*************************************************************************************************************************************

Title: Meta-regression of anti-CCP generation 

Date created: 03 August 2020
Date modified: 26 september 2021

NOTE: Comments begin with * or a block of text begins with /* and closes with */
Stata is case sensitive so if you are not familiar with it beware when you create variables and type commands and program names. 
Commands are expected to be lowercase. Also be careful with "=" and "==". For example, after the if command, Stata expects "==" 
for a test of equality; "=" produces an error in this case.

***********************************************************************************************************************************/

/* Set your working directory to the appropriate drive where you saved the file "anti-ccp.csv". 
Replace "U:\Handbook 2020" with your path.*/
cd "U:\Handbook 2020"

**************** 1. IMPORT DATA *********************

*** Read in the data from the .csv file. ***
insheet using "anti-ccp.csv", comma clear

*** Produce a summary of the dataset to check data import was ok. ***
describe 

/*************** 2. SET UP THE DATA BEFORE THE META-ANALYSIS 
Generate 5 new variables of type long. We need these before we can reshape the data.
•	n1 is number diseased
•	n0 is number without disease
•	true1 is number of true positives
•	true0 is the number of true negatives
•	study is the unique identifier for each study. _n will generate a sequence of numbers.***/
gen long n1=tp+fn
gen long n0=fp+tn
gen long true1=tp
gen long true0=tn
gen long recordid= _n 

*** Convert data from wide to long form. ***
reshape long n true, i(recordid) j(sens)

*** Generate a new binary variable spec of type byte that takes the value 0 when sens=1 and vice versa. ***				
gen byte spec=1-sens

*** Sort data to ensure studies are clustered together first by study.	***
sort studyid generation


**************** 3. META-REGRESSION  - USING meqrlogit WITH A COVARIATE **************************			

*** Create dummy variables for the covariate anti-CCP generation. ***
gen se1=0
gen sp1=0
gen se2=0
gen sp2=0
replace se1=1 if generation=="CCP1" & sens==1
replace sp1=1 if generation=="CCP1" & spec==1
replace se2=1 if generation=="CCP2" & sens==1
replace sp2=1 if generation=="CCP2" & spec==1

/*** Perform meta-analysis for each generation of anti-CCP and examine the variances of the random effects. 
Are they similar so that common/equal variances can be assumed in the meta-regression model or are they very different? ***/

*** Perform meta-analysis for CCP1 ***
meqrlogit true sens spec if generation=="CCP1", nocons|| studyid: sens spec, /// 	
nocons cov(un) binomial(n) refineopts(iterate(3)) intpoints(5) variance 

*** Perform meta-analysis for CCP2 ***
meqrlogit true sens spec if generation=="CCP2", nocons|| studyid: sens spec, /// 	
nocons cov(un) binomial(n) refineopts(iterate(3)) intpoints(5) variance 


/*** PART A: ANALYSES WITH COVARIATE GENERATION ASSUMING EQUAL VARIANCES ***/

*** Fit the model without the covariate. ***
meqrlogit true sens spec, nocons || studyid: sens spec, nocons cov(un) ///
binomial(n)  refineopts(iterate(3)) intpoints(5) variance nolr

*** Store the estimates of the log likelihood from the model above for doing the likeilhood ratio test later. ***
estimates store A

/*** Add covariate terms to the model for both logit sensitivity and logit specificity. 
This model assumes equal variances of the random effects for the logit sensitivities 
(and similarly for that of the logit specificities) for both generations of anti-CCP. ***/
meqrlogit true se1 se2 sp1 sp2, nocons || studyid: sens spec, nocons cov(un) ///
binomial(n)  refineopts(iterate(3)) intpoints(5) variance nolr

estimates store B

/* Perform a likelihood ratio test comparing model (A) without covariate with model (B) that includes the 
covariate generation and assumes equal variances. Use the stored values in A and B. */
lrtest A B

*** Assume sensitivity is the same for both generations but allow specificity to vary with anti-CCP generation. ***
meqrlogit true sens sp1 sp2, nocons || studyid: sens spec, nocons cov(un) binomial(n)  ///
refineopts(iterate(3)) intpoints(5) variance nolr

estimates store C

/* Perform a likelihood ratio test comparing model (C) that assumes sensitivity is the same for both generations
but allows specificity to vary with generation with model (B) that allows both sensitivity and specificity 
to vary with anti-CCP generation. Use the stored values in B and C. */
lrtest B C 

*** Assume specificity is the same for both generations but allow sensitivity to vary with anti-CCP generation. ***
meqrlogit true se1 se2 spec, nocons || studyid: sens spec, nocons cov(un) binomial(n)  ///
refineopts(iterate(3)) intpoints(5) variance nolr

estimates store D

/* Perform a likelihood ratio test comparing model (D) that assumes specificity is the same but allows sensitivity to vary with 
generation with model (B) that allows both sensitivity and specificity to vary with anti-CCP generation. 
Use the stored values in B and D. */
lrtest B D 

*** Run model B (model with covariate terms for both sensitivity and specificity) again. ***
meqrlogit true se1 se2 sp1 sp2, nocons || studyid: sens spec, nocons cov(un) binomial(n)  ///
refineopts(iterate(3)) intpoints(5) variance nolr

/* To find the covariance between the expected (mean) logit sensitivity and expected logit specificity, 
display contents of the variance-covariance matrix. */
matrix list e(V)  

*** Delete the program from Stata's memory if it exists already. ***
capture program drop renamematrix

*** Rename the elements of the coefficient and variance matrices. ***
program define renamematrix, eclass
	matrix mb = e(b)
	matrix mv = e(V)
	matrix colnames mb = logitse1:_cons logitse2:_cons logitsp1:_cons logitsp2:_cons 
	matrix colnames mv = logitse1:_cons logitse2:_cons logitsp1:_cons logitsp2:_cons 
	matrix rownames mv = logitse1:_cons logitse2:_cons logitsp1:_cons logitsp2:_cons 
	ereturn post mb mv
end

*** Run the program. ***
renamematrix
 
*** Display summary estimates by taking the inverse logits of the  mean logit sensitivity and mean logit specificity for each test. ***
_diparm logitse1, label(Sensitivity CCP1) invlogit 
_diparm logitse2, label(Sensitivity CCP2) invlogit 
_diparm logitsp1, label(Specificity CCP1) invlogit 
_diparm logitsp2, label(Specificity CCP2) invlogit 

*** Display other summary estimates derived using functions of the mean logit sensitivities and mean logit specificities. ***	
_diparm logitse1 logitsp1, label(LR+ CCP1) ci(log) function(invlogit(@1)/(1-invlogit(@2))) derivative(exp(@2-1)*invlogit(@1)^2/invlogit(@2) exp(@2)*invlogit(@1))
_diparm logitse2 logitsp2, label(LR+ CCP2) ci(log) function(invlogit(@1)/(1-invlogit(@2))) derivative(exp(@2-1)*invlogit(@1)^2/invlogit(@2) exp(@2)*invlogit(@1))
_diparm logitse1 logitsp1, label(LR- CCP1) ci(log) function((1-invlogit(@1))/invlogit(@2)) derivative(exp(-@1)*invlogit(@1)^2/invlogit(@2)  exp(-@1-@2)*invlogit(@1))  
_diparm logitse2 logitsp2, label(LR- CCP2) ci(log) function((1-invlogit(@1))/invlogit(@2)) derivative(exp(-@1)*invlogit(@1)^2/invlogit(@2)  exp(-@1-@2)*invlogit(@1)) 


/*** PART B: ANALYSES WITH COVARIATE GENERATION ALLOWING FOR UNEQUAL VARIANCES ***/

/* As noted in the "Analysing and presenting results" chapter, the variances of the random effects were assumed to be the same 
for logit(sensitivity) and logit(specificity) for both generations of the test in the above analyses. This assumption can be 
investigated by fitting additional models as follows. A likelihood ratio test can be used to assess if separate variances are needed. */

*** Model with equal variances ***
meqrlogit true se1 se2 sp1 sp2, nocons || studyid: sens spec, nocons cov(un) ///
binomial(n)  refineopts(iterate(3)) intpoints(5) variance nolr

estimates store A

*** Assumption of equal variances for logit sensitivities and logit specificities may not be tenable so allow separate variances. ***
meqrlogit true se1 se2 sp1 sp2, nocons || studyid: se1 sp1, nocons cov(un) ///
|| studyid: se2 sp2, nocons cov(un) binomial(n)  refineopts(iterate(3)) intpoints(5) variance nolr

estimates store B

lrtest A B


