
/********************************************************************************************
*																							*
* TITLE:			Comparative meta-analysis of diagnostic test accuracy studies			*
* AUTHOR:			Yemisi Takwoingi														*
* DATE CREATED:		01/10/2013								   								*
* DATE MODIFIED:	26/09/2021								   								*
* PURPOSE: 			Bivariate comparative meta-analysis of CT and MRI for diagnosis of CAD	*
*									 														*
*																							*
********************************************************************************************/

/* Set your working directory to the appropriate drive where you saved the file schuetz.csv. 
Replace "U:\Handbook 2020" with the appropriate path for you. */
cd "U:\Handbook 2020"

**************************** 1. IMPORT DATA **************************************

*** Read in the data from the .csv file. ***
insheet using "schuetz.csv", comma clear

*** Produce a summary of the dataset to check data import was ok ***
describe 


************************** 2. SET UP THE DATA BEFORE THE META-ANALYSIS **************

/* Generate 5 new variables of type long. We need these before we can reshape the data.
•	n1 is number diseased
•	n0 is number without disease
•	true1 is number of true positives
•	true0 is the number of true negatives
•	study is the unique identifier for each study (and test if a study evaluated more than one test). _n will generate a sequence of numbers.*/

gen long n1=tp+fn
gen long n0=fp+tn
gen long true1=tp
gen long true0=tn
gen long recordid= _n

*** Reshape the data from wide to long format. ***
reshape long n true, i(recordid) j(sens)

*** Sort data by study and test variables to cluster the records per study together. ***
gen byte spec=1-sens
sort study_id test


**************** 3. TEST COMPARISON - META-REGRESSION APPROACH **************************			

*** Create dummy variables for the covariate test. ***
gen seCT=0
gen spCT=0
gen seMRI=0
gen spMRI=0
replace seCT=1 if test=="CT" & sens==1
replace spCT=1 if test=="CT" & spec==1
replace seMRI=1 if test=="MRI" & sens==1
replace spMRI=1 if test=="MRI" & spec==1
														
*** Meta-analysis of CT ***
meqrlogit true sens spec if test=="CT", nocons ||study_id: sens spec, nocons cov(un) binomial(n) refineopts(iterate(3)) intpoints(5) variance 

*** Meta-analysis of MRI ***
meqrlogit true sens spec if test=="MRI", nocons ||study_id: sens spec, nocons cov(un) binomial(n) refineopts(iterate(3)) intpoints(5) variance 


/*** PART I: ANALYSES WITH COVARIATE TEST ASSUMING EQUAL VARIANCES ***/

*** Fit the model without the covariate. ***
meqrlogit true sens spec, nocons ||study_id: sens spec, nocons cov(un) binomial(n) refineopts(iterate(3)) intpoints(5) variance nolr

*** Store the estimates of the log likelihood from the model above for doing the likelIhood ratio test later. ***
estimates store A

*** Add covariate terms to the model for both logit sensitivity and logit specificity. This model assumes equal variances for both tests. ***
meqrlogit true seCT seMRI spCT spMRI, nocons ||study_id: sens spec, nocons cov(un) binomial(n) refineopts(iterate(3)) intpoints(5) variance nolr

estimates store B

/* Perform a likelihood ratio test comparing the model (A) without covariate with the model (B) that includes the covariate test and 
assumes equal variances for each test. Use the stored values in A and B. */
lrtest A B

*** Assume sensitivity is the same for CT and MRI but allow specificity to vary with test type. ***
meqrlogit true sens spCT spMRI, nocons ||study_id: sens spec, nocons cov(un) binomial(n) refineopts(iterate(3)) intpoints(5) variance nolr

estimates store C

/* Perform a likelihood ratio test comparing the model (C) that assumes sensitivity is the same for CT and MRI but allows specificity to vary with test type 
with the model (B) that allows both sensitivity and specificity to vary with test. Use the stored values in B and C. */
lrtest B C 

*** Assume specificity is the same for CT and MRI but allow sensitivity to vary with test type. ***
meqrlogit true seCT seMRI spec, nocons ||study_id: sens spec, nocons cov(un) binomial(n) refineopts(iterate(3)) intpoints(5) variance nolr

estimates store D

/* Perform a likelihood ratio test comparing the model (D) that assumes specificity is the same for CT and MRI but allows sensitivity to vary with test type 
with the model (B) that allows both sensitivity and specificity to vary with test. Use the stored values in B and D. */
lrtest B D 


/*** PART II: ANALYSES WITH COVARIATE TEST AND UNEQUAL VARIANCES ***/

/* Having looked at the variances assumption of equal variances may not be appropriate.
There was a marked difference in the variances for the logit specificities in the 2 meta-analyses so
let's fit a model with separate variances for the logits for each test */

*** Fit model (E) with covariate test and separate variances for the tests ***
meqrlogit true seCT seMRI spCT spMRI, nocons ||study_id: seCT spCT, nocons cov(un) ||study_id: seMRI spMRI, nocons cov(un) binomial(n) refineopts(iterate(3)) intpoints(5) variance nolr

estimates store E

/* Perform a likelihood ratio test comparing the model (B) that assumes equal variances with the model (E) that allows for separate variances for 
each test. Use the stored values in B and E. */
lrtest B E

/* Perform a likelihood ratio test comparing the model (A) without covariate  with the model (E) that includes the covariate test 
and allows for separate variances for each test. Use the stored values in A and E. */
lrtest A E

/* To find the covariance between the estimated mean logit sensitivity and mean logit specificity for each test, 
display contents of the variance-covariance matrix. */
matrix list e(V)  

/*** The nlcom command in Stata post-estimation of the regression model. The nlcom command uses nonlinear combinations 
of the parameter estimates to compute point estimates and their standard errors are computed using the delta method. ***/

*** To compute absolute differences ***
nlcom diff_sensitivity: invlogit(_b[seCT])-invlogit(_b[seMRI])
nlcom diff_specificity: invlogit(_b[spCT])-invlogit(_b[spMRI])

*** To compute ratios ***
nlcom log_relative_sensitivity: log(invlogit(_b[seCT]))-log(invlogit(_b[seMRI]))
nlcom log_relative_specificity: log(invlogit(_b[spCT]))-log(invlogit(_b[spMRI]))

*** Delete the program from Stata's memory if it exists already. ***
capture program drop renamematrix

*** Rename the elements of the coefficient and variance matrices. ***
program define renamematrix, eclass
	matrix mb = e(b)
	matrix mv = e(V)
	matrix colnames mb = logitseCT:_cons logitseMRI:_cons logitspCT:_cons logitspMRI:_cons 
	matrix colnames mv = logitseCT:_cons logitseMRI:_cons logitspCT:_cons logitspMRI:_cons 
	matrix rownames mv = logitseCT:_cons logitseMRI:_cons logitspCT:_cons logitspMRI:_cons 
	ereturn post mb mv
end

*** Run the program ***
renamematrix
 
*** Display summary estimates by taking the inverse logits of the mean logit sensitivity and mean logit specificity for each test. ***
_diparm logitseCT, label(Sensitivity CT) invlogit 
_diparm logitseMRI, label(Sensitivity MRI) invlogit 
_diparm logitspCT, label(Specificity CT) invlogit 
_diparm logitspMRI, label(Specificity MRI) invlogit 

*** Display other summary estimates derived using functions of the mean logit sensitivities and mean logit specificities. ***	
_diparm logitseCT logitspCT, label(LR+ CT) ci(log) function(invlogit(@1)/(1-invlogit(@2))) derivative(exp(@2-@1)*invlogit(@1)^2/invlogit(@2) exp(@2)*invlogit(@1))
_diparm logitseMRI logitspMRI, label(LR+ MRI) ci(log) function(invlogit(@1)/(1-invlogit(@2))) derivative(exp(@2-@1)*invlogit(@1)^2/invlogit(@2) exp(@2)*invlogit(@1))
_diparm logitseCT logitspCT, label(LR- CT) ci(log) function((1-invlogit(@1))/invlogit(@2)) derivative(exp(-@1)*invlogit(@1)^2/invlogit(@2)  exp(-@1-@2)*invlogit(@1))  
_diparm logitseMRI logitspMRI, label(LR- MRI) ci(log) function((1-invlogit(@1))/invlogit(@2)) derivative(exp(-@1)*invlogit(@1)^2/invlogit(@2)  exp(-@1-@2)*invlogit(@1)) 



*******************************************Additional analyses ***************************************************************

*** Fit model (F) assuming same sensitivity but separate variances for the tests. ***
meqrlogit true sens spCT spMRI, nocons || study: seCT spCT, nocons cov(un) || study: seMRI spMRI, nocons cov(un) binomial(n) refineopts(iterate(3)) intpoints(5) variance nolr

estimates store F

/* Perform a likelihood ratio test comparing the model (F) that assumes the same sensitivity with the model (E) that allows for the effect of testtype. 
Use the stored values in F and E. */
lrtest F E

*** Fit model (G) assuming same specificity but separate variances for the tests. ***
meqrlogit true seCT seMRI spec, nocons || study: seCT spCT, nocons cov(un) || study: seMRI spMRI, nocons cov(un) binomial(n) refineopts(iterate(3)) intpoints(5) variance nolr

estimates store G

/* Perform a likelihood ratio test comparing the model (G) that assumes the same specificity with the model (E) that allows for the effect of testtype. 
Use the stored values in G and E. */
lrtest G E


******************************** DIRECT COMPARISON *****************************

*** Fit the model for each test. ***
meqrlogit true sens spec if indirect==0 & test =="CT", nocons ||study_id: sens spec, nocons cov(un) binomial(n) refineopts(iterate(3)) intpoints(5) stddev nolr gradient

meqrlogit true sens spec if indirect==0 & test=="MRI", nocons ||study_id: sens spec, nocons cov(un) binomial(n) refineopts(iterate(3)) intpoints(5) stddev nolr gradient

*** Since the correlation parameter was poorly estimated for CT, fit a model without this parameter by specifying an independent variance-covariance structure (using option cov(ind)). ***
meqrlogit true sens spec if indirect==0 & test=="CT", nocons ||study_id: sens spec, nocons cov(ind) binomial(n) refineopts(iterate(3)) intpoints(5) stddev nolr gradient

*** Fit the model without the covariate. ***
*** Add gradient option to display gradient values. ***
meqrlogit true sens spec if indirect==0, nocons ||study_id: sens spec, nocons cov(un) binomial(n) refineopts(iterate(3)) intpoints(5) stddev nolr gradient
meqrlogit true sens spec if indirect==0, nocons ||study_id: sens spec, nocons cov(un) binomial(n) refineopts(iterate(3)) intpoints(5) variance nolr gradient

*** Store the estimates of the log likelihood from the model above for doing the likelihood ratio test later. ***
estimates store X

*** Add covariate terms to the model for both logit sensitivity and logit specificity. This model assumes equal variances for both tests. ***
meqrlogit true seCT seMRI spCT spMRI if indirect==0, nocons ||study_id: sens spec, nocons cov(un) binomial(n) refineopts(iterate(3)) intpoints(5) variance nolr gradient

estimates store Y

/* Perform a likelihood ratio test comparing the model (A) without covariate with the model (B) that includes the covariate test and 
assumes equal variances for each test. Use the stored values in A and B. */
lrtest X Y

*** Fit model (Z) with covariate test and separate variances for the tests. ***
meqrlogit true seCT seMRI spCT spMRI if indirect==0, nocons ||study_id: seCT spCT, nocons cov(un) ||study_id: seMRI spMRI, nocons cov(un) binomial(n)  refineopts(iterate(3)) intpoints(5) stddev nolr gradient

estimates store Z

lrtest X Y
lrtest Y Z




