/********************************************************************************************************************************

Title: Meta-analysis of anti-CCP for rheumatoid arthritis

Date created: 22 June 2020
Date modified: 26 September 2021

NOTE: Comments begin with * or a block of text begins with /* and closes with */
Stata is case sensitive so if you are not familiar with it beware when you create variables and type commands and program names. 
Commands are expected to be lowercase. Also be careful with "=" and "==". For example, after the if command, Stata expects "==" 
for a test of equality; "=" produces an error in this case.

*******************************************************************************************************************************/

************************* 1. IMPORT DATA *****************************
*** Read in the data from the .csv file. Note you should change the file path to where you saved the anti-CCP.csv file ***
insheet using "U:\Handbook 2020\anti-CCP.csv", comma clear

*** Produce a summary of the dataset to check data import was ok ***
describe 

************************ 2. SET UP THE DATA ****************************

/*** Generate 5 new variables of type long. We need these before we can reshape the data.
•	n1 is number diseased
•	n0 is number without disease
•	true1 is number of true positives
•	true0 is the number of true negatives
•	study is the unique identifier for each study. _n will generate a sequence of numbers.
IMPORTANT NOTE: after modifying the data, if you need to run metandi again, remember to use the original data. ***/
gen long n1=tp+fn
gen long n0=fp+tn
gen long true1=tp
gen long true0=tn
gen long recordid= _n 

*** Convert data from wide to long form ***
reshape long n true, i(recordid) j(sens)

*** Generate a new binary variable spec of type byte that takes the value 0 when sens=1 and vice versa ***														
gen byte spec=1-sens

*** Sort data to ensure studies are clustered together first by study	***
sort studyid


******************* 3. PERFORM META-ANALYSIS USING GLLAMM *****************

/*** Numeric variable expected for option i() so encode studyid and generate a new numeric variable named id. 
Since this analysis is not a test comparison and there are no comparative studies, recordid and id are essentially
the same. However, to avoid confusion if this code is later modified for a test comparison, id was created and 
used instead of recordid. ***/
encode studyid, gen(id)

*** create the gllamm equations ***
eq eq1: sens
eq eq0: spec

*** See the help file for a description of the available options for gllamm and the syntax. ***
gllamm true sens spec, nocons i(id) nrf(2) eqs(eq1 eq0) ///
family(binomial) link(logit) denom(n) ip(g) nip(5) adapt

/* To find the covariance between the expected (mean) logit sensitivity and expected logit specificity, 
display contents of the variance-covariance matrix: */
matrix list e(V)  
       

*** DISPLAY SUMMARY ESTIMATES ***
*** Drop the program in case it is already in Stata’s memory. *** 
capture program drop renamematrix

/* Write a little program renaming elements of the coefficient and variance matrices */
program define renamematrix, eclass
matrix mb = e(b)
matrix mv = e(V)
matrix colnames mb = logitse:_cons logitsp:_cons 
matrix colnames mv = logitse:_cons logitsp:_cons 
matrix rownames mv = logitse:_cons logitsp:_cons 
ereturn post mb mv
end

*** Run the program ***
renamematrix

*** Display summary points ***
_diparm logitse, label(Sensitivity) invlogit 
_diparm logitsp, label(Specificity) invlogit 
_diparm logitse logitsp, label(LR+) ci(log) f(invlogit(@1)/(1-invlogit(@2))) d( exp(@2-1)*invlogit(@1)^2/invlogit(@2) exp(@2)*invlogit(@1))
_diparm logitse logitsp, label(LR-) ci(log) f((1-invlogit(@1))/invlogit(@2)) d( exp(-@1)*invlogit(@1)^2/invlogit(@2)  exp(-@1-@2)*invlogit(@1))  
_diparm logitse logitsp, label(DOR) ci(log) f(exp(@1+@2)) d(exp(@1+@2) exp(@1+@2))


