

/************************************************************************************
*																					*
* TITLE:			Univariate random effects logistic regression for meta-analysis *
*					of test accuracy studies using the meqrlogit command			*
* AUTHOR:			Yemisi Takwoingi												*
* DATE CREATED:		31/08/2013								   						*
* DATE MODIFIED:	26/09/2021							   							*
* PURPOSE: 			Comparative meta-analysis of 9 T1 serum tests for Down's 		*
*					syndrome screening												*
*									 												*
*																					*
************************************************************************************/

/* Set your working directory to the appropriate drive where you saved the file "Nine T1 serum test strategies.csv". 
Replace "U:\Handbook 2020" with your path.*/
cd "U:\Handbook 2020"

**************** 1. IMPORT DATA *********************

*** Read in the data from the .csv file. ***
insheet using "Nine T1 serum test strategies.csv", comma clear

*** Produce a summary of the dataset to check data import was ok. ***
describe 

/*** test is currently a string variable so for ease of selecting a test, 
create a new numeric variable t and then list the contents of t. ***/
encode test, gen(t)
label list t

*** Tabulate the number of studies for each test. ***
table t


************** 2. SET UP THE DATA 	********************

*** Create new variables and reshape the data. ***
gen long n1=tp+fn
gen long n0=fp+tn
gen long true1=tp
gen long true0=tn
gen record_id = _n
reshape long n true, i(record_id) j(sens)
sort record_id sens
gen byte spec=1-sens
egen study= group(studyid)

*** Set up dummy variables for the covariate test. ***
gen se1=0
gen se2=0
gen se3=0
gen se4=0
gen se5=0
gen se6=0
gen se7=0
gen se8=0
gen se9=0

replace se1=1 if t==1 & sens==1
replace se2=1 if t==2 & sens==1
replace se3=1 if t==3 & sens==1
replace se4=1 if t==4 & sens==1
replace se5=1 if t==5 & sens==1
replace se6=1 if t==6 & sens==1
replace se7=1 if t==7 & sens==1
replace se8=1 if t==8 & sens==1
replace se9=1 if t==9 & sens==1


************** 3. UNIVARIATE META-REGRESSION 	********************

*** Model without covariate ***
meqrlogit true sens, nocons || study: sens, nocons cov(un) ///
binomial(n)  refineopts(iterate(3)) intpoints(5) variance nolr

estimates store A

*** Model with covariate terms and equal variances ***
meqrlogit true se1 se2 se3 se4 se5 se6 se7 se8 se9, nocons || study: ///
sens, nocons binomial(n)  refineopts(iterate(3)) intpoints(5) variance nolr

estimates store B

*** Model with covariate terms and separate variances ***
meqrlogit true  se1 se2 se3 se4 se5 se6 se7 se8 se9, nocons ///
|| study: se1, nocons || study: se2, nocons || study: se3, nocons ///
|| study: se4, nocons || study: se5, nocons || study: se6, nocons ///
|| study: se7, nocons || study: se8, nocons || study: se9, nocons ///
binomial(n)  refineopts(iterate(6)) intpoints(1) variance nolr

estimates store C

*** Perform likelihood ratio tests to compare models. ***
lrtest A B  
lrtest B C
lrtest A C

*** Display contents of the covariance matrix. ***
matrix list e(V)        

/*** Write a little program renaming elements of the coefficient and variance matrices.
Drop the program first though in case it is already in Stata’s memory. ***/
capture program drop renamematrix

program define renamematrix, eclass
	matrix mb = e(b)
	matrix mv = e(V)
	matrix colnames mb = logitse1:_cons logitse2:_cons logitse3:_cons logitse4:_cons logitse5:_cons logitse6:_cons logitse7:_cons logitse8:_cons logitse9:_cons 
	matrix colnames mv = logitse1:_cons logitse2:_cons logitse3:_cons logitse4:_cons logitse5:_cons logitse6:_cons logitse7:_cons logitse8:_cons logitse9:_cons 
	matrix rownames mv = logitse1:_cons logitse2:_cons logitse3:_cons logitse4:_cons logitse5:_cons logitse6:_cons logitse7:_cons logitse8:_cons logitse9:_cons 
	ereturn post mb mv
end

*** Run the program. ***
renamematrix

*** Display summary points by taking the inverse logits of the mean logit sensitivity for each type. ***
_diparm logitse1, label(Sensitivity 1) invlogit 
_diparm logitse2, label(Sensitivity 2) invlogit 
_diparm logitse3, label(Sensitivity 3) invlogit 
_diparm logitse4, label(Sensitivity 4) invlogit 
_diparm logitse5, label(Sensitivity 5) invlogit 
_diparm logitse6, label(Sensitivity 6) invlogit
_diparm logitse7, label(Sensitivity 7) invlogit 
_diparm logitse8, label(Sensitivity 8) invlogit 
_diparm logitse9, label(Sensitivity 9) invlogit 





